import { CommandLineAction } from './CommandLineAction';
/**
 * @public
 */
export declare class DynamicCommandLineAction extends CommandLineAction {
    protected onDefineParameters(): void;
    protected onExecute(): Promise<void>;
}
//# sourceMappingURL=DynamicCommandLineAction.d.ts.map