Die TimeLimit-Installation von dem Gerät <%- deviceName %> hat eine Manipulation gemeldet.
Sie können die Details dazu in der TimeLimit-App einsehen.

Solange Sie nicht die Warnung in TimeLimit bestätigen (ignorieren und/ oder beheben)
erhalten Sie keine weitere Manipulationsbenachrichtigungen für dieses Gerät.


TimeLimit at <%- deviceName %> reported a manipulation.
You can see the details in the TimeLimit App.

You will not get any further warnings for this device
until you confirm (ignore and/ or fix) the manipulation in the TimeLimit App.

----------------------

Sie erhalten diese Nachricht, weil Sie E-Mail-Benachrichtigungen in TimeLimit aktiviert haben.
Sie können die Nachrichten in der TimeLimit-App abbestellen.
Falls Sie Fragen haben können Sie einfach auf diese E-Mail antworten.

You got this mail because you enable mail notifications for this in TimeLimit.
You can disable getting these messages in the TimeLimit App.
If you have got any questions, then you can reply to this message.

<C> <%- mailimprint %>
