"use strict";
/*
 * server component for the TimeLimit App
 * Copyright (C) 2019 - 2020 Jonas Lochmann
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.dispatchParentAction = exports.dispatchChildAction = exports.dispatchAppLogicAction = void 0;
var app_logic_action_1 = require("./app-logic-action");
Object.defineProperty(exports, "dispatchAppLogicAction", { enumerable: true, get: function () { return app_logic_action_1.dispatchAppLogicAction; } });
var child_action_1 = require("./child-action");
Object.defineProperty(exports, "dispatchChildAction", { enumerable: true, get: function () { return child_action_1.dispatchChildAction; } });
var parent_action_1 = require("./parent-action");
Object.defineProperty(exports, "dispatchParentAction", { enumerable: true, get: function () { return parent_action_1.dispatchParentAction; } });
//# sourceMappingURL=index.js.map