"use strict";
// imported from https://github.com/DefinitelyTyped/DefinitelyTyped/blob/b83cf9ef8b044e69f05b2a00aa7c6cb767a9acd2/types/cookie/index.d.ts (now deleted)
/**
 * Basic HTTP cookie parser and serializer for HTTP servers.
 */
Object.defineProperty(exports, "__esModule", { value: true });
