"use strict";
/*
 * server component for the TimeLimit App
 * Copyright (C) 2019 - 2026 Jonas Lochmann
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateCategoryFlagsAction = exports.UpdateCategoryDisableLimitsAction = exports.UpdateCategoryBlockedTimesAction = exports.UpdateCategoryBlockAllNotificationsAction = exports.UpdateCategoryBatteryLimitAction = exports.UpdateAppActivitiesAction = exports.TriedDisablingDeviceAdminAction = exports.SignOutAtDeviceAction = exports.SetUserTimezoneAction = exports.SetUserDisableLimitsUntilAction = exports.SetSendDeviceConnected = exports.SetRelaxPrimaryDeviceAction = exports.SetParentCategoryAction = exports.SetKeepSignedInAction = exports.SetDeviceUserAction = exports.SetDeviceDefaultUserTimeoutAction = exports.SetDeviceDefaultUserAction = exports.SetConsiderRebootManipulationAction = exports.SetChildPasswordAction = exports.SetCategoryForUnassignedAppsAction = exports.SetCategoryExtraTimeAction = exports.ReportU2fLoginAction = exports.ReplyToKeyRequestAction = exports.RenameChildAction = exports.ResetCategoryNetworkIdsAction = exports.RemoveUserAction = exports.RemoveParentU2fKeyAction = exports.RemoveInstalledAppsAction = exports.RemoveCategoryAppsAction = exports.IncrementCategoryExtraTimeAction = exports.IgnoreManipulationAction = exports.ForceSyncAction = exports.FinishKeyRequestAction = exports.DeleteTimeLimitRuleAction = exports.DeleteCategoryAction = exports.CreateTimeLimitRuleAction = exports.CreateCategoryAction = exports.ChildSignInAction = exports.ChildChangePasswordAction = exports.ChangeParentPasswordAction = exports.AddUsedTimeActionVersion2 = exports.AddUsedTimeAction = exports.AddParentU2fKeyAction = exports.AddInstalledAppsAction = exports.AddUserAction = exports.AddCategoryNetworkIdAction = exports.AddCategoryAppsAction = exports.ParentAction = exports.ChildAction = exports.AppLogicAction = void 0;
exports.SendKeyRequestAction = exports.UploadDevicePublicKeyAction = exports.UpdateUserLimitLoginPreBlockDuration = exports.ReviewChildTaskAction = exports.UpdateChildTaskAction = exports.DeleteChildTaskAction = exports.PingAction = exports.MarkTaskPendingAction = exports.UpdateUserLimitLoginCategory = exports.UpdateUserFlagsAction = exports.UpdateTimelimitRuleAction = exports.UpdateParentNotificationFlagsAction = exports.UpdateNetworkTimeVerificationAction = exports.UpdateInstalledAppsAction = exports.UpdateEnableActivityLevelBlockingAction = exports.UpdateDeviceStatusAction = exports.UpdateDeviceNameAction = exports.UpdateCategoryTitleAction = exports.UpdateCategoryTimeWarningsAction = exports.UpdateCategoryTemporarilyBlockedAction = exports.UpdateCategorySortingAction = void 0;
var basetypes_1 = require("./basetypes");
Object.defineProperty(exports, "AppLogicAction", { enumerable: true, get: function () { return basetypes_1.AppLogicAction; } });
Object.defineProperty(exports, "ChildAction", { enumerable: true, get: function () { return basetypes_1.ChildAction; } });
Object.defineProperty(exports, "ParentAction", { enumerable: true, get: function () { return basetypes_1.ParentAction; } });
var addcategoryapps_1 = require("./addcategoryapps");
Object.defineProperty(exports, "AddCategoryAppsAction", { enumerable: true, get: function () { return addcategoryapps_1.AddCategoryAppsAction; } });
var addcategorynetworkid_1 = require("./addcategorynetworkid");
Object.defineProperty(exports, "AddCategoryNetworkIdAction", { enumerable: true, get: function () { return addcategorynetworkid_1.AddCategoryNetworkIdAction; } });
var adduser_1 = require("./adduser");
Object.defineProperty(exports, "AddUserAction", { enumerable: true, get: function () { return adduser_1.AddUserAction; } });
var addinstalledapps_1 = require("./addinstalledapps");
Object.defineProperty(exports, "AddInstalledAppsAction", { enumerable: true, get: function () { return addinstalledapps_1.AddInstalledAppsAction; } });
var addu2fkey_1 = require("./addu2fkey");
Object.defineProperty(exports, "AddParentU2fKeyAction", { enumerable: true, get: function () { return addu2fkey_1.AddParentU2fKeyAction; } });
var addusedtime_1 = require("./addusedtime");
Object.defineProperty(exports, "AddUsedTimeAction", { enumerable: true, get: function () { return addusedtime_1.AddUsedTimeAction; } });
var addusedtime2_1 = require("./addusedtime2");
Object.defineProperty(exports, "AddUsedTimeActionVersion2", { enumerable: true, get: function () { return addusedtime2_1.AddUsedTimeActionVersion2; } });
var changeparentpassword_1 = require("./changeparentpassword");
Object.defineProperty(exports, "ChangeParentPasswordAction", { enumerable: true, get: function () { return changeparentpassword_1.ChangeParentPasswordAction; } });
var childchangepassword_1 = require("./childchangepassword");
Object.defineProperty(exports, "ChildChangePasswordAction", { enumerable: true, get: function () { return childchangepassword_1.ChildChangePasswordAction; } });
var childsignin_1 = require("./childsignin");
Object.defineProperty(exports, "ChildSignInAction", { enumerable: true, get: function () { return childsignin_1.ChildSignInAction; } });
var createcategory_1 = require("./createcategory");
Object.defineProperty(exports, "CreateCategoryAction", { enumerable: true, get: function () { return createcategory_1.CreateCategoryAction; } });
var createtimelimitrule_1 = require("./createtimelimitrule");
Object.defineProperty(exports, "CreateTimeLimitRuleAction", { enumerable: true, get: function () { return createtimelimitrule_1.CreateTimeLimitRuleAction; } });
var deletecategory_1 = require("./deletecategory");
Object.defineProperty(exports, "DeleteCategoryAction", { enumerable: true, get: function () { return deletecategory_1.DeleteCategoryAction; } });
var deletetimelimitrule_1 = require("./deletetimelimitrule");
Object.defineProperty(exports, "DeleteTimeLimitRuleAction", { enumerable: true, get: function () { return deletetimelimitrule_1.DeleteTimeLimitRuleAction; } });
var finishkeyrequest_1 = require("./finishkeyrequest");
Object.defineProperty(exports, "FinishKeyRequestAction", { enumerable: true, get: function () { return finishkeyrequest_1.FinishKeyRequestAction; } });
var forcesync_1 = require("./forcesync");
Object.defineProperty(exports, "ForceSyncAction", { enumerable: true, get: function () { return forcesync_1.ForceSyncAction; } });
var ignoremanipulation_1 = require("./ignoremanipulation");
Object.defineProperty(exports, "IgnoreManipulationAction", { enumerable: true, get: function () { return ignoremanipulation_1.IgnoreManipulationAction; } });
var incrementcategoryextratime_1 = require("./incrementcategoryextratime");
Object.defineProperty(exports, "IncrementCategoryExtraTimeAction", { enumerable: true, get: function () { return incrementcategoryextratime_1.IncrementCategoryExtraTimeAction; } });
var removecategoryapps_1 = require("./removecategoryapps");
Object.defineProperty(exports, "RemoveCategoryAppsAction", { enumerable: true, get: function () { return removecategoryapps_1.RemoveCategoryAppsAction; } });
var removeinstalledapps_1 = require("./removeinstalledapps");
Object.defineProperty(exports, "RemoveInstalledAppsAction", { enumerable: true, get: function () { return removeinstalledapps_1.RemoveInstalledAppsAction; } });
var removeu2fkey_1 = require("./removeu2fkey");
Object.defineProperty(exports, "RemoveParentU2fKeyAction", { enumerable: true, get: function () { return removeu2fkey_1.RemoveParentU2fKeyAction; } });
var removeuser_1 = require("./removeuser");
Object.defineProperty(exports, "RemoveUserAction", { enumerable: true, get: function () { return removeuser_1.RemoveUserAction; } });
var resetcategorynetworkids_1 = require("./resetcategorynetworkids");
Object.defineProperty(exports, "ResetCategoryNetworkIdsAction", { enumerable: true, get: function () { return resetcategorynetworkids_1.ResetCategoryNetworkIdsAction; } });
var renamechild_1 = require("./renamechild");
Object.defineProperty(exports, "RenameChildAction", { enumerable: true, get: function () { return renamechild_1.RenameChildAction; } });
var replytokeyrequest_1 = require("./replytokeyrequest");
Object.defineProperty(exports, "ReplyToKeyRequestAction", { enumerable: true, get: function () { return replytokeyrequest_1.ReplyToKeyRequestAction; } });
var reportu2flogin_1 = require("./reportu2flogin");
Object.defineProperty(exports, "ReportU2fLoginAction", { enumerable: true, get: function () { return reportu2flogin_1.ReportU2fLoginAction; } });
var setcategoryextratime_1 = require("./setcategoryextratime");
Object.defineProperty(exports, "SetCategoryExtraTimeAction", { enumerable: true, get: function () { return setcategoryextratime_1.SetCategoryExtraTimeAction; } });
var setcategoryforunassignedapps_1 = require("./setcategoryforunassignedapps");
Object.defineProperty(exports, "SetCategoryForUnassignedAppsAction", { enumerable: true, get: function () { return setcategoryforunassignedapps_1.SetCategoryForUnassignedAppsAction; } });
var setchildpassword_1 = require("./setchildpassword");
Object.defineProperty(exports, "SetChildPasswordAction", { enumerable: true, get: function () { return setchildpassword_1.SetChildPasswordAction; } });
var setconsiderrebootmanipulation_1 = require("./setconsiderrebootmanipulation");
Object.defineProperty(exports, "SetConsiderRebootManipulationAction", { enumerable: true, get: function () { return setconsiderrebootmanipulation_1.SetConsiderRebootManipulationAction; } });
var setdevicedefaultuser_1 = require("./setdevicedefaultuser");
Object.defineProperty(exports, "SetDeviceDefaultUserAction", { enumerable: true, get: function () { return setdevicedefaultuser_1.SetDeviceDefaultUserAction; } });
var setdevicedefaultusertimeout_1 = require("./setdevicedefaultusertimeout");
Object.defineProperty(exports, "SetDeviceDefaultUserTimeoutAction", { enumerable: true, get: function () { return setdevicedefaultusertimeout_1.SetDeviceDefaultUserTimeoutAction; } });
var setdeviceuser_1 = require("./setdeviceuser");
Object.defineProperty(exports, "SetDeviceUserAction", { enumerable: true, get: function () { return setdeviceuser_1.SetDeviceUserAction; } });
var setkeepsignedin_1 = require("./setkeepsignedin");
Object.defineProperty(exports, "SetKeepSignedInAction", { enumerable: true, get: function () { return setkeepsignedin_1.SetKeepSignedInAction; } });
var setparentcategory_1 = require("./setparentcategory");
Object.defineProperty(exports, "SetParentCategoryAction", { enumerable: true, get: function () { return setparentcategory_1.SetParentCategoryAction; } });
var setrelaxprimarydevice_1 = require("./setrelaxprimarydevice");
Object.defineProperty(exports, "SetRelaxPrimaryDeviceAction", { enumerable: true, get: function () { return setrelaxprimarydevice_1.SetRelaxPrimaryDeviceAction; } });
var setsenddeviceconnected_1 = require("./setsenddeviceconnected");
Object.defineProperty(exports, "SetSendDeviceConnected", { enumerable: true, get: function () { return setsenddeviceconnected_1.SetSendDeviceConnected; } });
var setuserdisablelimitsuntil_1 = require("./setuserdisablelimitsuntil");
Object.defineProperty(exports, "SetUserDisableLimitsUntilAction", { enumerable: true, get: function () { return setuserdisablelimitsuntil_1.SetUserDisableLimitsUntilAction; } });
var setusertimezone_1 = require("./setusertimezone");
Object.defineProperty(exports, "SetUserTimezoneAction", { enumerable: true, get: function () { return setusertimezone_1.SetUserTimezoneAction; } });
var signoutatdevice_1 = require("./signoutatdevice");
Object.defineProperty(exports, "SignOutAtDeviceAction", { enumerable: true, get: function () { return signoutatdevice_1.SignOutAtDeviceAction; } });
var trieddisablingdeviceadmin_1 = require("./trieddisablingdeviceadmin");
Object.defineProperty(exports, "TriedDisablingDeviceAdminAction", { enumerable: true, get: function () { return trieddisablingdeviceadmin_1.TriedDisablingDeviceAdminAction; } });
var updateappactivities_1 = require("./updateappactivities");
Object.defineProperty(exports, "UpdateAppActivitiesAction", { enumerable: true, get: function () { return updateappactivities_1.UpdateAppActivitiesAction; } });
var updatecategorybatterylimit_1 = require("./updatecategorybatterylimit");
Object.defineProperty(exports, "UpdateCategoryBatteryLimitAction", { enumerable: true, get: function () { return updatecategorybatterylimit_1.UpdateCategoryBatteryLimitAction; } });
var updatecategoryblockallnotifications_1 = require("./updatecategoryblockallnotifications");
Object.defineProperty(exports, "UpdateCategoryBlockAllNotificationsAction", { enumerable: true, get: function () { return updatecategoryblockallnotifications_1.UpdateCategoryBlockAllNotificationsAction; } });
var updatecategoryblockedtimes_1 = require("./updatecategoryblockedtimes");
Object.defineProperty(exports, "UpdateCategoryBlockedTimesAction", { enumerable: true, get: function () { return updatecategoryblockedtimes_1.UpdateCategoryBlockedTimesAction; } });
var updatecategorydisablelimits_1 = require("./updatecategorydisablelimits");
Object.defineProperty(exports, "UpdateCategoryDisableLimitsAction", { enumerable: true, get: function () { return updatecategorydisablelimits_1.UpdateCategoryDisableLimitsAction; } });
var updatecategoryflags_1 = require("./updatecategoryflags");
Object.defineProperty(exports, "UpdateCategoryFlagsAction", { enumerable: true, get: function () { return updatecategoryflags_1.UpdateCategoryFlagsAction; } });
var updatecategorysorting_1 = require("./updatecategorysorting");
Object.defineProperty(exports, "UpdateCategorySortingAction", { enumerable: true, get: function () { return updatecategorysorting_1.UpdateCategorySortingAction; } });
var updatecategorytemporarilyblocked_1 = require("./updatecategorytemporarilyblocked");
Object.defineProperty(exports, "UpdateCategoryTemporarilyBlockedAction", { enumerable: true, get: function () { return updatecategorytemporarilyblocked_1.UpdateCategoryTemporarilyBlockedAction; } });
var updatecategorytimewarnings_1 = require("./updatecategorytimewarnings");
Object.defineProperty(exports, "UpdateCategoryTimeWarningsAction", { enumerable: true, get: function () { return updatecategorytimewarnings_1.UpdateCategoryTimeWarningsAction; } });
var updatecategorytitle_1 = require("./updatecategorytitle");
Object.defineProperty(exports, "UpdateCategoryTitleAction", { enumerable: true, get: function () { return updatecategorytitle_1.UpdateCategoryTitleAction; } });
var updatedevicename_1 = require("./updatedevicename");
Object.defineProperty(exports, "UpdateDeviceNameAction", { enumerable: true, get: function () { return updatedevicename_1.UpdateDeviceNameAction; } });
var updatedevicestatus_1 = require("./updatedevicestatus");
Object.defineProperty(exports, "UpdateDeviceStatusAction", { enumerable: true, get: function () { return updatedevicestatus_1.UpdateDeviceStatusAction; } });
var updateenableactivitylevelblocking_1 = require("./updateenableactivitylevelblocking");
Object.defineProperty(exports, "UpdateEnableActivityLevelBlockingAction", { enumerable: true, get: function () { return updateenableactivitylevelblocking_1.UpdateEnableActivityLevelBlockingAction; } });
var updateinstalledapps_1 = require("./updateinstalledapps");
Object.defineProperty(exports, "UpdateInstalledAppsAction", { enumerable: true, get: function () { return updateinstalledapps_1.UpdateInstalledAppsAction; } });
var updatenetworktimeverification_1 = require("./updatenetworktimeverification");
Object.defineProperty(exports, "UpdateNetworkTimeVerificationAction", { enumerable: true, get: function () { return updatenetworktimeverification_1.UpdateNetworkTimeVerificationAction; } });
var updateparentnotificationflags_1 = require("./updateparentnotificationflags");
Object.defineProperty(exports, "UpdateParentNotificationFlagsAction", { enumerable: true, get: function () { return updateparentnotificationflags_1.UpdateParentNotificationFlagsAction; } });
var updatetimelimitrule_1 = require("./updatetimelimitrule");
Object.defineProperty(exports, "UpdateTimelimitRuleAction", { enumerable: true, get: function () { return updatetimelimitrule_1.UpdateTimelimitRuleAction; } });
var updateuserflags_1 = require("./updateuserflags");
Object.defineProperty(exports, "UpdateUserFlagsAction", { enumerable: true, get: function () { return updateuserflags_1.UpdateUserFlagsAction; } });
var updateuserlimitlogincategory_1 = require("./updateuserlimitlogincategory");
Object.defineProperty(exports, "UpdateUserLimitLoginCategory", { enumerable: true, get: function () { return updateuserlimitlogincategory_1.UpdateUserLimitLoginCategory; } });
var marktaskpendingaction_1 = require("./marktaskpendingaction");
Object.defineProperty(exports, "MarkTaskPendingAction", { enumerable: true, get: function () { return marktaskpendingaction_1.MarkTaskPendingAction; } });
var ping_1 = require("./ping");
Object.defineProperty(exports, "PingAction", { enumerable: true, get: function () { return ping_1.PingAction; } });
var deletechildtaskaction_1 = require("./deletechildtaskaction");
Object.defineProperty(exports, "DeleteChildTaskAction", { enumerable: true, get: function () { return deletechildtaskaction_1.DeleteChildTaskAction; } });
var updatechildtaskaction_1 = require("./updatechildtaskaction");
Object.defineProperty(exports, "UpdateChildTaskAction", { enumerable: true, get: function () { return updatechildtaskaction_1.UpdateChildTaskAction; } });
var reviewchildtaskaction_1 = require("./reviewchildtaskaction");
Object.defineProperty(exports, "ReviewChildTaskAction", { enumerable: true, get: function () { return reviewchildtaskaction_1.ReviewChildTaskAction; } });
var updateuserlimitloginpreblockduration_1 = require("./updateuserlimitloginpreblockduration");
Object.defineProperty(exports, "UpdateUserLimitLoginPreBlockDuration", { enumerable: true, get: function () { return updateuserlimitloginpreblockduration_1.UpdateUserLimitLoginPreBlockDuration; } });
var uploaddevicepublickey_1 = require("./uploaddevicepublickey");
Object.defineProperty(exports, "UploadDevicePublicKeyAction", { enumerable: true, get: function () { return uploaddevicepublickey_1.UploadDevicePublicKeyAction; } });
var sendkeyrequest_1 = require("./sendkeyrequest");
Object.defineProperty(exports, "SendKeyRequestAction", { enumerable: true, get: function () { return sendkeyrequest_1.SendKeyRequestAction; } });
//# sourceMappingURL=index.js.map