"use strict";
/*
 * The MIT License
 *
 * Copyright (c) Paul Crawford
 * Copyright (c) 2020 Jonas Lochmann
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IABVerifier = void 0;
const crypto_1 = require("crypto");
const ALGORITHM = 'RSA-SHA1';
class IABVerifier {
    constructor(publicKeyString) {
        this.publicKey = (0, crypto_1.createPublicKey)({
            key: Buffer.from(publicKeyString, 'base64'),
            format: 'der',
            type: 'spki'
        });
    }
    verifyReceipt(signedData, signature) {
        const verifier = (0, crypto_1.createVerify)(ALGORITHM);
        verifier.update(signedData);
        return verifier.verify(this.publicKey, signature, 'base64');
    }
}
exports.IABVerifier = IABVerifier;
//# sourceMappingURL=iab_verifierr.js.map