Sie haben eine Löschung Ihres Benutzerkontos angefordert. Diese wurde soeben durchgeführt.

You requested the deletion of your account. This deletion is finished now.

----------------------

Sie erhalten diese Nachricht aufgrund Ihrer Anfrage.
Falls Sie Fragen haben können Sie einfach auf diese E-Mail antworten.

You got this mail due to your request.
If you have got any questions, then you can reply to this message.

<C> <%- mailimprint %>
