/**
 * Helpers for working with the ts-command-line API.
 *
 * @public
 */
export declare class CommandLineHelper {
    /**
     * Returns true if the current command line action is tab-complete.
     *
     * @public
     */
    static isTabCompletionActionRequest(argv: string[]): boolean;
}
//# sourceMappingURL=CommandLineHelper.d.ts.map