import * as argparse from 'argparse';
export declare class CommandLineParserExitError extends Error {
    readonly exitCode: number;
    constructor(exitCode: number, message: string);
}
export declare class CustomArgumentParser extends argparse.ArgumentParser {
    exit(status: number, message: string): void;
    error(err: Error | string): void;
}
//# sourceMappingURL=CommandLineParserExitError.d.ts.map