/**
 * Some parameter types can receive their values from an environment variable instead of
 * a command line argument. This class provides some utility methods for parsing environment
 * variable values.
 * @internal
 */
export declare class EnvironmentVariableParser {
    static parseAsList(envVarName: string): string[] | undefined;
}
//# sourceMappingURL=EnvironmentVariableParser.d.ts.map