"use strict";
/*
 * server component for the TimeLimit App
 * Copyright (C) 2019 Jonas Lochmann
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseParentAction = exports.parseChildAction = exports.parseAppLogicAction = void 0;
var applogicaction_1 = require("./applogicaction");
Object.defineProperty(exports, "parseAppLogicAction", { enumerable: true, get: function () { return applogicaction_1.parseAppLogicAction; } });
var childaction_1 = require("./childaction");
Object.defineProperty(exports, "parseChildAction", { enumerable: true, get: function () { return childaction_1.parseChildAction; } });
var parentaction_1 = require("./parentaction");
Object.defineProperty(exports, "parseParentAction", { enumerable: true, get: function () { return parentaction_1.parseParentAction; } });
//# sourceMappingURL=index.js.map