import { CommandLineParser } from './CommandLineParser';
/**
 * @public
 */
export declare class DynamicCommandLineParser extends CommandLineParser {
    protected onDefineParameters(): void;
}
//# sourceMappingURL=DynamicCommandLineParser.d.ts.map