"use strict";
/*
 * server component for the TimeLimit App
 * Copyright (C) 2019 - 2026 Jonas Lochmann
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPingModel = exports.attributes = exports.types = void 0;
const Sequelize = require("sequelize");
const columns_1 = require("./columns");
exports.types = {
    ping: 1,
    pong: 2,
    all: [1, 2]
};
exports.attributes = {
    familyId: Object.assign(Object.assign({}, columns_1.familyIdColumn), { primaryKey: true }),
    receiverDeviceId: Object.assign(Object.assign({}, columns_1.idWithinFamilyColumn), { primaryKey: true }),
    senderDeviceId: Object.assign(Object.assign({}, columns_1.idWithinFamilyColumn), { primaryKey: true }),
    type: {
        type: Sequelize.INTEGER,
        allowNull: false,
        validate: {
            isIn: [exports.types.all]
        },
        primaryKey: true
    },
    token: Object.assign({}, columns_1.idWithinFamilyColumn)
};
const createPingModel = (sequelize) => sequelize.define('Ping', exports.attributes);
exports.createPingModel = createPingModel;
//# sourceMappingURL=ping.js.map