'use strict';

// TODO [>=7]: remove me. I've been moved to 'exports' in package.json

/**
  * A Sequelize module that contains the sequelize entry point.
  *
  * @module sequelize
  */

/** Exports the sequelize entry point. */
module.exports = require('./lib');
