/**
 * String constants for command line processing.
 *
 * @public
 */
export declare const enum CommandLineConstants {
    /**
     * The name of the built-in action that serves suggestions for tab-completion
     */
    TabCompletionActionName = "tab-complete"
}
export declare const SCOPING_PARAMETER_GROUP: unique symbol;
//# sourceMappingURL=Constants.d.ts.map