<%- introtext %>

<%- code %>

<%- outrotext %>

<% if (deviceName !== null) { -%>
<%- deviceNameIntro %> <%- deviceName %> <%- deviceNameOutro %>

<% } -%>
----------------------

Sie erhalten diese Nachricht, da Ihre E-Mail-Adresse in der TimeLimit-App zum Anmelden eingegeben wurde.
Wenn Sie diese E-Mail nicht angefordert haben, können Sie diese ignorieren.
Falls Sie Fragen haben können Sie einfach auf diese E-Mail antworten.

You got this mail because your mail address was entered at the TimeLimit App for signing in.
If you did not request this mail, then you can ignore it.
If you have got any questions, then you can reply to this message.

<C> <%- mailimprint %>
