<%- preText %> <%- deviceName %> <%- postText %>

<%- securityText %>

----------------------

Sie erhalten diese Nachricht, da ein Gerät zu Ihrem Konto hinzugefügt wurde.
Mit diesem Gerät können die TimeLimit-Einstellungen ohne ein Passwort geändert werden.
Falls Sie Fragen haben können Sie einfach auf diese E-Mail antworten.

You got this message because a device was added to your account.
This device can be used to change the TimeLimit configuration without any password.
If you have got any questions, then you can reply to this message.

<C> <%- mailimprint %>
