<%- text %>

<%- securityText %>

----------------------

Sie erhalten diese Nachricht, da Ihr TimeLimit-Passwort geändert wurde.
Falls Sie Fragen haben können Sie einfach auf diese E-Mail antworten.

You got this message because the password reset feature was used for your account.
If you have got any questions, then you can reply to this message.

<C> <%- mailimprint %>
